SystemOrganization addCategory: #'Seaside-Gettext-Core'!

WAObject subclass: #WALocalizationContext
	instanceVariableNames: 'domain translator localeId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Gettext-Core'!

!WALocalizationContext methodsFor: 'accessing' stamp: 'pmm 5/8/2010 21:41'!
defaultApplicationName
	^ 'defaultApplication'! !

!WALocalizationContext methodsFor: 'accessing' stamp: 'pmm 5/8/2010 22:13'!
domain
	domain isNil ifTrue: [
		"determine the domain name
		1. try if it is configured on the application
		2. use the application name
		3. send #defaultApplicationName"
		| configuredDomain |
		configuredDomain := [ self application gettextDomain ]
			on: WAAttributeNotFound
			do: [ :error | nil ].
		domain := configuredDomain isEmptyOrNil
			ifFalse: [ configuredDomain ]
			ifTrue: [
				| applicationName |
				applicationName := self application name.
				applicationName isEmptyOrNil
					ifFalse: [ applicationName ]
					ifTrue: [ self defaultApplicationName ] ] ].
	^ domain! !

!WALocalizationContext methodsFor: 'accessing' stamp: 'pmm 5/8/2010 22:05'!
localeId
	localeId isNil ifTrue: [
		| session |
		session := self session.
		self session notNil
			ifTrue: [ localeId := session locale asLocaleID ]
			ifFalse: [
				self requestContext acceptLanguage do: [ :each |
					| candidate |
					candidate := each asLocaleID.
					GetTextTranslator translations
						at: candidate
						ifPresent: [ :t |
							translator := t.
							^ localeId := candidate ].
					"if the locale has a country try with just the language"
					each country isNil ifTrue: [
						candidate := each withoutCountry asLocaleID.
						GetTextTranslator translations
							at: candidate
							ifPresent: [ :t |
								translator := t.
								^ localeId := candidate ] ] ].
				"fall back to default"
				translator := NaturalLanguageTranslator default.
				localeId := translator ] ].
	^ localeId! !

!WALocalizationContext methodsFor: 'translating' stamp: 'pmm 5/8/2010 21:50'!
translate: aString
	^ self translator
		translate: aString
		inDomain: self domain! !

!WALocalizationContext methodsFor: 'translating' stamp: 'pmm 5/8/2010 21:47'!
translate: aString to: aLocale
	^ (GetTextTranslator availableForLocaleID: aLocale asLocaleID)
		translate: aString
		inDomain: self domain! !

!WALocalizationContext methodsFor: 'accessing' stamp: 'pmm 5/8/2010 22:05'!
translator
	translator isNil ifTrue: [
		translator := GetTextTranslator availableForLocaleID: self localeId ].
	^ translator! !

WAObject subclass: #WATranslatedArgumentsFinder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Gettext-Core'!

!WATranslatedArgumentsFinder methodsFor: 'accessing' stamp: 'pmm 5/8/2010 21:22'!
translatedArguments
	"self new translatedArguments"
	| searcher currentClass result matchHandler |
	currentClass := WAValueHolder with: nil.
	result := GROrderedMultiMap new.
	matchHandler := [ :node :answer |
		| firstArgument |
		firstArgument := node arguments first.
		firstArgument isLiteral ifTrue: [
			| key |
			key := MethodReference
				class: currentClass contents
				selector: node methodNode selector. 
			result at: key add: firstArgument value ] ].
	searcher := RBParseTreeSearcher new
		matches: '`@object translate: `#string' do: matchHandler;
		matches: '`@object translate: `#string to: `@arg' do: matchHandler;
		yourself.
	BrowserEnvironment new classesAndSelectorsDo: [ :class :selector |
		(searcher canMatchMethod: class >> selector) ifTrue: [
		| tree |
		(tree := class parseTreeFor: selector) notNil ifTrue: [
			currentClass contents: class.
			searcher executeTree: tree ] ] ].
	^ Array streamContents: [ :stream |
		result keysDo: [ :key |
			stream nextPut: key -> (result allAt: key) ] ]! !

!WAApplication methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/8/2010 22:07'!
gettextDomain
	^ self preferenceAt: #gettextDomain! !

WADynamicVariable subclass: #WACurrentLocalizationContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Gettext-Core'!

WARequestFilter subclass: #WALocalizationContextFilter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Gettext-Core'!

!WALocalizationContextFilter methodsFor: 'processing' stamp: 'pmm 5/7/2010 17:33'!
handleFiltered: aRequestContext 
	WACurrentLocalizationContext
		use: WALocalizationContext new
		during: [ super handleFiltered: aRequestContext ]! !

!WASession methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/7/2010 17:36'!
defaultLocale
	^ WALocale
		language: 'en'
		country: 'US'! !

!WASession methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/7/2010 17:37'!
locale
	^ self properties
		at: #gettextLocale
		ifAbsentPut: [ self defaultLocale ]! !

!WASession methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/8/2010 20:57'!
locale: aLocale
	self properties
		at: #gettextLocale
		put: aLocale! !

WASystemConfiguration subclass: #WAGetTextConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Gettext-Core'!

!WAGetTextConfiguration methodsFor: 'description' stamp: 'pmm 5/8/2010 21:59'!
describeOn: config
	(config string: #gettextDomain)
		label: 'Gettext Domain';
		comment: 'The name of the domain where all lozalized strings should come from.'! !

!WALocale methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/7/2010 17:46'!
asLocaleID
	^ LocaleID
		isoLanguage: self language
		isoCountry: self country! !

!GRPackage class methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/8/2010 15:42'!
seasideGettextCore
	^ self new
		name: 'Seaside-Gettext-Core';
		description: 'i18n and l10n using gettext';
		url: #seasideAddonsUrl;
		yourself! !

GetTextExporter subclass: #WAGetTextExporter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Gettext-Core'!

!WAGetTextExporter methodsFor: 'exporting' stamp: 'pmm 5/8/2010 13:52'!
appendStringArtumentsInto: domains
	| found |
	
	found := WATranslatedArgumentsFinder new translatedArguments.
	found do: [ :assoc |
		| literals domainName methodReference keywords |
		methodReference := assoc key.
		keywords := assoc value.
		domainName := self getTextDomainForClassCategory:
			(Smalltalk at: methodReference classSymbol) category.
		literals := domains at: domainName ifAbsentPut: [ Dictionary new ].
		keywords do: [ :literal |
			| references |
			references := literals at: literal ifAbsentPut: [ OrderedCollection new ].
			references add: methodReference ] ]. 
! !

!WAGetTextExporter methodsFor: 'exporting' stamp: 'pmm 5/8/2010 13:52'!
appendTranslations: domains
	self appendStringArtumentsInto: domains! !

!WARenderer methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/8/2010 16:09'!
translate: aString
	self text: (WACurrentLocalizationContext value translate: aString)! !

!WARenderer methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/8/2010 16:09'!
translate: aString to: aLocale
	self text: (WACurrentLocalizationContext value translate: aString to: aLocale)! !
